# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gymir_result.proto
# Protobuf Python Version: 4.25.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x12gymir_result.proto\x12\x0bgymirResult"g\n\x0bGymirResult\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x19\n\x11\x61uxiliary_load_kw\x18\x02 \x01(\x01\x12/\n\x06result\x18\x03 \x03(\x0b\x32\x1f.gymirResult.SimulationInstance"\xd3\x04\n\x12SimulationInstance\x12\x0f\n\x07\x65poch_s\x18\x01 \x01(\x01\x12\x11\n\ttask_type\x18\x02 \x01(\t\x12\x11\n\ttask_name\x18\x03 \x01(\t\x12\x14\n\x0clatitude_deg\x18\x04 \x01(\x01\x12\x15\n\rlongitude_deg\x18\x05 \x01(\x01\x12\x13\n\x0bheading_deg\x18\x06 \x01(\x01\x12!\n\x19wave_height_significant_m\x18\x07 \x01(\x01\x12\x1a\n\x12wave_peak_period_s\x18\x08 \x01(\x01\x12\x1e\n\x16wave_dir_rel_north_deg\x18\t \x01(\x01\x12\x1f\n\x17wave_dir_rel_vessel_deg\x18\n \x01(\x01\x12\x16\n\x0ewind_speed_mps\x18\x0b \x01(\x01\x12\x1e\n\x16wind_dir_rel_north_deg\x18\x0c \x01(\x01\x12\x1f\n\x17wind_dir_rel_vessel_deg\x18\r \x01(\x01\x12\x16\n\x0eweather_source\x18\x0e \x01(\t\x12\x1c\n\x14speed_over_ground_kn\x18\x0f \x01(\x01\x12\x1e\n\x16speed_through_water_kn\x18\x10 \x01(\x01\x12\x19\n\x11\x63urrent_speed_mps\x18\x11 \x01(\x01\x12!\n\x19\x63urrent_dir_rel_north_deg\x18\x12 \x01(\x01\x12\x10\n\x08power_kw\x18\x13 \x01(\x01\x12\x13\n\x0btorque_k_nm\x18\x14 \x01(\x01\x12\x12\n\nthrust_k_n\x18\x15 \x01(\x01\x12\x1c\n\x14total_resistance_k_n\x18\x16 \x01(\x01"c\n\x17PropulsionPowerInstance\x12\x0f\n\x07\x65poch_s\x18\x01 \x01(\x01\x12\x1b\n\x13propulsion_power_kw\x18\x02 \x01(\x01\x12\x1a\n\x12\x61uxiliary_power_kw\x18\x03 \x01(\x01"K\n\x15OperationProfilePoint\x12\x0f\n\x07\x65poch_s\x18\x01 \x01(\x01\x12\x10\n\x08speed_kn\x18\x02 \x01(\x01\x12\x0f\n\x07\x64raft_m\x18\x03 \x01(\x01"\xb8\x01\n\x10TimeSeriesResult\x12I\n\x1bpropulsion_power_timeseries\x18\x01 \x03(\x0b\x32$.gymirResult.PropulsionPowerInstance\x12\x1a\n\x12\x61uxiliary_power_kw\x18\x02 \x01(\x01\x12=\n\x11operation_profile\x18\x03 \x03(\x0b\x32".gymirResult.OperationProfilePointb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "gymir_result_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_GYMIRRESULT"]._serialized_start = 35
    _globals["_GYMIRRESULT"]._serialized_end = 138
    _globals["_SIMULATIONINSTANCE"]._serialized_start = 141
    _globals["_SIMULATIONINSTANCE"]._serialized_end = 736
    _globals["_PROPULSIONPOWERINSTANCE"]._serialized_start = 738
    _globals["_PROPULSIONPOWERINSTANCE"]._serialized_end = 837
    _globals["_OPERATIONPROFILEPOINT"]._serialized_start = 839
    _globals["_OPERATIONPROFILEPOINT"]._serialized_end = 914
    _globals["_TIMESERIESRESULT"]._serialized_start = 917
    _globals["_TIMESERIESRESULT"]._serialized_end = 1101
# @@protoc_insertion_point(module_scope)
