# Autogenerated by nbdev

d = {
    "settings": {
        "branch": "master",
        "doc_baseurl": "/MachSysS/",
        "doc_host": "https://keviny.github.io",
        "git_url": "https://SintefOceanEnergySystem@dev.azure.com/SintefOceanEnergySystem/MachinerySystemStucture/_git/MachinerySystemStucture",
        "lib_path": "MachSysS",
    },
    "syms": {
        "MachSysS.convert_feems_result_to_proto": {
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter": (
                "convertfeemsresulttoproto.html#feemsresultconverter",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter.__init__": (
                "convertfeemsresulttoproto.html#feemsresultconverter.__init__",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter._get_feems_result_proto_for_subsystem": (
                "convertfeemsresulttoproto.html#feemsresultconverter._get_feems_result_proto_for_subsystem",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter._retrieve_time_series_data_from_components": (
                "convertfeemsresulttoproto.html#feemsresultconverter._retrieve_time_series_data_from_components",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter._time_interval_to_time_array": (
                "convertfeemsresulttoproto.html#feemsresultconverter._time_interval_to_time_array",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter.electric_system": (
                "convertfeemsresulttoproto.html#feemsresultconverter.electric_system",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter.get_feems_result_proto": (
                "convertfeemsresulttoproto.html#feemsresultconverter.get_feems_result_proto",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter.get_timeseries_for_power_sources_and_energy_storage": (
                "convertfeemsresulttoproto.html#feemsresultconverter.get_timeseries_for_power_sources_and_energy_storage",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter.isSystemElectric": (
                "convertfeemsresulttoproto.html#feemsresultconverter.issystemelectric",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter.mechanical_system": (
                "convertfeemsresulttoproto.html#feemsresultconverter.mechanical_system",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
            "MachSysS.convert_feems_result_to_proto.FEEMSResultConverter.system_feems": (
                "convertfeemsresulttoproto.html#feemsresultconverter.system_feems",
                "MachSysS/convert_feems_result_to_proto.py",
            ),
        },
        "MachSysS.convert_gymir_result_to_proto": {
            "MachSysS.convert_gymir_result_to_proto.GymirResultConverter": (
                "convertgymirresulttoproto.html#gymirresultconverter",
                "MachSysS/convert_gymir_result_to_proto.py",
            ),
            "MachSysS.convert_gymir_result_to_proto.GymirResultConverter.__init__": (
                "convertgymirresulttoproto.html#gymirresultconverter.__init__",
                "MachSysS/convert_gymir_result_to_proto.py",
            ),
            "MachSysS.convert_gymir_result_to_proto.GymirResultConverter.read_csv": (
                "convertgymirresulttoproto.html#gymirresultconverter.read_csv",
                "MachSysS/convert_gymir_result_to_proto.py",
            ),
            "MachSysS.convert_gymir_result_to_proto.GymirResultConverter.to_csv": (
                "convertgymirresulttoproto.html#gymirresultconverter.to_csv",
                "MachSysS/convert_gymir_result_to_proto.py",
            ),
            "MachSysS.convert_gymir_result_to_proto.find_csv_name_from_message_field_name": (
                "convertgymirresulttoproto.html#find_csv_name_from_message_field_name",
                "MachSysS/convert_gymir_result_to_proto.py",
            ),
            "MachSysS.convert_gymir_result_to_proto.get_message_type_for_simulation_instance": (
                "convertgymirresulttoproto.html#get_message_type_for_simulation_instance",
                "MachSysS/convert_gymir_result_to_proto.py",
            ),
        },
        "MachSysS.convert_proto_timeseries": {
            "MachSysS.convert_proto_timeseries.convert_proto_timeseries_to_pd_dataframe": (
                "converttimeseriesresultprototopandas.html#convert_proto_timeseries_to_pd_dataframe",
                "MachSysS/convert_proto_timeseries.py",
            ),
            "MachSysS.convert_proto_timeseries.convert_proto_timeseries_to_pd_series": (
                "converttimeseriesresultprototopandas.html#convert_proto_timeseries_to_pd_series",
                "MachSysS/convert_proto_timeseries.py",
            ),
        },
        "MachSysS.convert_to_feems": {
            "MachSysS.convert_to_feems.collect_electric_components_from_sub_system": (
                "converttofeems.html#collect_electric_components_from_sub_system",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_emission_curve_to_feems": (
                "converttofeems.html#convert_emission_curve_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_feems_shaftlines_to_feems_mechanical_propulsion_system": (
                "converttofeems.html#convert_feems_shaftlines_to_feems_mechanical_propulsion_system",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_feems_switchboards_to_feems_electric_power_system": (
                "converttofeems.html#convert_feems_switchboards_to_feems_electric_power_system",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_generic_electric_subsystem_to_feems": (
                "converttofeems.html#convert_generic_electric_subsystem_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_nox_calculation_method": (
                "converttofeems.html#convert_nox_calculation_method",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_battery_system_to_feems": (
                "converttofeems.html#convert_proto_battery_system_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_battery_to_feems": (
                "converttofeems.html#convert_proto_battery_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_cogas_to_feems": (
                "converttofeems.html#convert_proto_cogas_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_coges_to_feems": (
                "converttofeems.html#convert_proto_coges_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_curve1d_to_np_array": (
                "converttofeems.html#convert_proto_curve1d_to_np_array",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_efficiency_bsfc_power_to_np_array": (
                "converttofeems.html#convert_proto_efficiency_bsfc_power_to_np_array",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_electric_component_to_feems": (
                "converttofeems.html#convert_proto_electric_component_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_electric_machine_to_feems": (
                "converttofeems.html#convert_proto_electric_machine_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_electric_system_to_feems": (
                "converttofeems.html#convert_proto_electric_system_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_engine_to_feems": (
                "converttofeems.html#convert_proto_engine_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_fuel_cell_system_to_feems": (
                "converttofeems.html#convert_proto_fuel_cell_system_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_generic_electric_component_to_feems": (
                "converttofeems.html#convert_proto_generic_electric_component_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_genset_to_feems": (
                "converttofeems.html#convert_proto_genset_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_mechanical_system_to_feems": (
                "converttofeems.html#convert_proto_mechanical_system_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_point_to_list": (
                "converttofeems.html#convert_proto_point_to_list",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_propulsion_system_to_feems": (
                "converttofeems.html#convert_proto_propulsion_system_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_pti_pto_subsystem_to_feems": (
                "converttofeems.html#convert_proto_pti_pto_subsystem_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_serial_subsystem_to_feems": (
                "converttofeems.html#convert_proto_serial_subsystem_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_shaftline_to_feems": (
                "converttofeems.html#convert_proto_shaftline_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_supercapacitor_system_to_feems": (
                "converttofeems.html#convert_proto_supercapacitor_system_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_supercapacitor_to_feems": (
                "converttofeems.html#convert_proto_supercapacitor_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.convert_proto_switchboard_to_feems": (
                "converttofeems.html#convert_proto_switchboard_to_feems",
                "MachSysS/convert_to_feems.py",
            ),
            "MachSysS.convert_to_feems.get_component_type": (
                "converttofeems.html#get_component_type",
                "MachSysS/convert_to_feems.py",
            ),
        },
        "MachSysS.convert_to_protobuf": {
            "MachSysS.convert_to_protobuf.convert_battery_component_to_protobuf": (
                "converttoprotobuf.html#convert_battery_component_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_bsfc_curve_to_protobuf": (
                "converttoprotobuf.html#convert_bsfc_curve_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_cogas_component_to_protobuf": (
                "converttoprotobuf.html#convert_cogas_component_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_efficiency_curve_to_protobuf": (
                "converttoprotobuf.html#convert_efficiency_curve_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_electric_component_to_protobuf": (
                "converttoprotobuf.html#convert_electric_component_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_electric_machine_to_protobuf": (
                "converttoprotobuf.html#convert_electric_machine_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_electric_system_to_protobuf": (
                "converttoprotobuf.html#convert_electric_system_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_electric_system_to_protobuf_machinery_system": (
                "converttoprotobuf.html#convert_electric_system_to_protobuf_machinery_system",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_emission_curves_to_protobuf": (
                "converttoprotobuf.html#convert_emission_curves_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_engine_component_to_protobuf": (
                "converttoprotobuf.html#convert_engine_component_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_hybrid_propulsion_system_to_protobuf": (
                "converttoprotobuf.html#convert_hybrid_propulsion_system_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_mechanical_propulsion_system_with_electric_system_to_protobuf": (
                "converttoprotobuf.html#convert_mechanical_propulsion_system_with_electric_system_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_mechanical_system_to_protobuf": (
                "converttoprotobuf.html#convert_mechanical_system_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_nox_calculation_method_to_protobuf": (
                "converttoprotobuf.html#convert_nox_calculation_method_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_np_array_to_protobuf_power_curve": (
                "converttoprotobuf.html#convert_np_array_to_protobuf_power_curve",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_serial_electric_system_to_protobuf": (
                "converttoprotobuf.html#convert_serial_electric_system_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_shaftline_to_protobuf": (
                "converttoprotobuf.html#convert_shaftline_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_supercapacitor_component_to_protobuf": (
                "converttoprotobuf.html#convert_supercapacitor_component_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
            "MachSysS.convert_to_protobuf.convert_switchboard_to_protobuf": (
                "converttoprotobuf.html#convert_switchboard_to_protobuf",
                "MachSysS/convert_to_protobuf.py",
            ),
        },
        "MachSysS.feems_result_pb2": {},
        "MachSysS.gymir_result_pb2": {},
        "MachSysS.system_structure_pb2": {},
        "MachSysS.utility": {
            "MachSysS.utility.retrieve_gymir_result_from_file": (
                "utility.html#retrieve_gymir_result_from_file",
                "MachSysS/utility.py",
            ),
            "MachSysS.utility.retrieve_machinery_system_from_file": (
                "utility.html#retrieve_machinery_system_from_file",
                "MachSysS/utility.py",
            ),
        },
    },
}
